import re
import math
import logging
import requests
import ipaddress

from django.db.models import F
from django.db.models import Min, Max
from django.db import connections
from django.conf import settings
from django.contrib.postgres.aggregates import ArrayAgg

logger = logging.getLogger(__name__)


list_private_networks = [
    "0.0.0.0/8",
    "10.0.0.0/8",
    "100.64.0.0/10",
    "127.0.0.0/8",
    "169.254.0.0/16",
    "172.16.0.0/12",
    "192.0.0.0/24",
    "192.0.2.0/24",
    "192.88.99.0/24",
    "192.168.0.0/16",
    "198.18.0.0/15",
    "198.51.100.0/24",
    "203.0.113.0/24",
    "233.252.0.0/24",
    "::/128",
    "::1/128",
    "::ffff:0:0/96",
    "::ffff:0:0:0/96",
    "64:ff9b::/96",
    "64:ff9b:1::/48",
    "100::/64",
    "2001:0000::/32",
    "2001:20::/28",
    "2001:db8::/32",
    "2002::/16",
    "fc00::/7",
    "fe80::/10",
    "ff00::/8"
]

try:
    http_proxy = settings.HTTP_PROXY_CONF
    https_proxy = settings.HTTP_PROXY_CONF
    proxyDict = {
        "http": http_proxy,
        "https": https_proxy
    }
except AttributeError as err:
    proxyDict = {
        "http": None,
        "https": None
    }


def is_local(ip):
    try:
        ip = ipaddress.ip_address(ip)
        for network in list_private_networks:
            if ip in ipaddress.ip_network(network):
                return True
        return False
    except Exception:
        return False


def get_params(group):
    if group:
        params = {"body_equal": ArrayAgg('body'),
                  "referer_equal": ArrayAgg('referer'),
                  "method_equal": ArrayAgg('method'),
                  "url_equal": ArrayAgg('url'),
                  "param_equal": ArrayAgg('param'),
                  "ua_equal": ArrayAgg('ua'),
                  "cookie_equal": ArrayAgg('cookie'),
                  "timestamp__max": Max('timestamp'),
                  "timestamp__min": Min('timestamp'),
                  "timestamp": ArrayAgg('timestamp'),
                  "ip": ArrayAgg('ip'),
                  "id": Max('id')}
    else:
        params = {"body_equal": F('body'),
                  "referer_equal": F('referer'),
                  "method_equal": F('method'),
                  "url_equal": F('url'),
                  "param_equal": F('param'),
                  "ua_equal": F('ua'),
                  "cookie_equal": F('cookie'),
                  "timestamp__max": F('timestamp'),
                  "timestamp__min": F('timestamp'),
                  "timestamp": F('timestamp'),
                  "ip": F('ip'),
                  "id": F('id')}

    return params


def convert_size(size_bytes):
    if size_bytes == 0 or size_bytes is None:
        return "-"
    if size_bytes < 100:
        result = size_bytes * 8
        return '{result} b/s'.format(result=result)
    elif size_bytes < 10000:
        result = round(size_bytes * 0.0078125, 2)
        return '{result} Kb/s'.format(result=result)
    else:
        result = round(size_bytes * 7.6294 * math.pow(10, -6), 2)
        return '{result} Mb/s'.format(result=result)


def filter_upstreams(names, exp=None):
    ups = []
    if (exp is None or
            exp == '*' or
            exp == ''):
        ups = names
    else:
        for d in exp.split(', '):
            for u in names:
                if d[0] == "*":
                    if d[2:] == u:
                        ups.append(u)
                    if d[1:] in u:
                        ups.append(u)
                elif d == u:
                    ups.append(u)
        ups = set(ups)
    return ups


def get_update_time():
    headers = {'User-Agent': settings.USER_AGENT}
    try:
        r = requests.get(settings.URI_RL_TS, proxies=proxyDict, headers=headers)
        return int(r.text)
    except Exception as e:
        logger.error(
            'An error occurred while getting information about the timestamp of the rules. Error information: {err}'
            .format(err=e)
        )
        return None


def remove_zero_from_ip(ip):
    if type(ipaddress.ip_address(ip)) is ipaddress.IPv4Address:
        no_zero_ip = re.sub(r"\b0+(\d)", r'\1', str(ip))
    else:
        no_zero_ip = re.sub(r"\b0+(\d)", r'\1', str(ip))
    return no_zero_ip


def get_list_messages(storage):
    list_messages = list()
    for message in storage:
        mes = dict()
        mes['message'] = message.message
        mes['tags'] = message.tags
        mes['level'] = message.level
        list_messages.append(mes)
    return list_messages


def convert_to_punycode(text: str) -> str:
    num_cyrillic = find_cyrillic_letter(text)
    if num_cyrillic > 0:
        text = text[:num_cyrillic] + text[num_cyrillic:].encode('idna').decode()
    elif num_cyrillic == 0:
        text = text.encode('idna').decode()

    return text


def convert_from_punycode(text: str) -> str:
    if text.startswith('xn--'):
        text = text.encode().decode('idna')
    elif 'xn--' in text:
        text = text[:text.index('xn--')] + text[text.index('xn--'):].encode().decode('idna')

    return text


def find_cyrillic_letter(text):
    if re.search('[а-яА-Я]', text):
        return re.search('[а-яА-Я]', text).start()
    return -1


def check_db():
    try:
        is_valid = 'mt_export' in connections['waf'].introspection.table_names() \
                   and 'mt_main' in connections['waf'].introspection.table_names() \
                   and 'openapi_schemas' in connections['waf'].introspection.table_names()
    except Exception as e:
        logger.error(
            'An error occurred while checking the database. Check the connection to the database. Error Information: {}'
            .format(e)
        )
        is_valid = False

    return is_valid
