import re
import ast
import json
import base64
import logging
import requests
import multiprocessing

from functools import wraps
from netaddr import IPNetwork, IPRange, IPAddress, AddrFormatError, AddrConversionError

from django.conf import settings
from django.http import HttpResponseRedirect

from my_waf.default_values import brute, ddos, nwaf_limit
from main.models.wafuser import WAFUSER
from main.utils import convert_to_punycode, convert_from_punycode

logger = logging.getLogger(__name__)

try:
    api_url = settings.API_URI
except AttributeError:
    api_url = 'http://localhost:8080/nw-api/'

USER_AGENT = settings.USER_AGENT
logger = logging.getLogger(__name__)
auth_url = settings.URI_AUTH


def my_waf_access(function):
    @wraps(function)
    def wrap(request, *args, **kwargs):
        if request.user.is_anonymous:
            return HttpResponseRedirect('/')

        wafuser = WAFUSER.objects.get(id=request.user.id)
        if wafuser.ws_toggle and settings.API_URI:
            return function(request, *args, **kwargs)
        else:
            return HttpResponseRedirect('/')

    return wrap


def check_plans_for_tab_mlc(keys):
    for key in keys:
        if key.tarif != 'Light':
            return True
    return False


def get_proxy_dict():
    """
    Function to get a proxy dictionary
    :return: proxy dictionary
    """
    if settings.HTTP_PROXY_CONF == '':
        return None
    else:
        try:
            http_proxy = settings.HTTP_PROXY_CONF
            https_proxy = settings.HTTP_PROXY_CONF
            proxy_dict = {
                "http": http_proxy,
                "https": https_proxy
            }
        except AttributeError as e:
            logging.info('Failed to set proxy with error: {}'.format(str(e)))
            proxy_dict = {
                "http": None,
                "https": None
            }
        return proxy_dict


def get_waf_id(key):
    if isinstance(key, str):
        if key and len(key) > 6:
            format_key = 'xxx' + key[-6:]
        else:
            format_key = key
    else:
        format_key = key.get_formated_key()
        key = key.key
    try:
        response_check_key = requests.post(
            auth_url,
            proxies=get_proxy_dict(),
            headers={'User-agent': USER_AGENT},
            data={'key': key}
        )
        if response_check_key.ok:
            check_key_result = json.loads(response_check_key.text.replace('\'', '\"'))
            if check_key_result['status'] == 'success':
                return check_key_result['waf_id']
    except Exception as e:
        logger.error(
            'An error occurred while getting waf_id by key: {key}. Error information: {err}'
            .format(key=format_key, err=e)
        )
        return None


def get_settings(module, key):
    """
    Function to get settings
    :param module: Module name
    :param key: Sha256 license key
    :return: Received settings
    """
    url = ''
    if key and len(key) > 6:
        format_key = 'xxx' + key[-6:]
    else:
        format_key = key
    if module == 'MLC':
        url = '/get_mlc_settings'
    if module == 'DYN':
        url = '/get_dyn_settings'
    if module == 'ERL':
        url = '/get_dyn_erl?extended=yes'
    if module == 'WL':
        url = '/get_dyn_wl?extended=yes'
    if module == 'BL':
        url = '/get_dyn_bl?extended=yes'
    response = requests.post(
        api_url + url,
        headers={'User-agent': USER_AGENT},
        data={'key': key},
        timeout=10
    )
    if response.ok:
        if module == 'ERL' or module == 'WL':
            response_list = response.text.split('\n')
            get_rules = ast.literal_eval(response_list[0])
            if get_rules['status'] == 'success':
                return response_list[1:]
        elif module == 'BL':
            response_list = response.text.split('\n')
            get_rules = ast.literal_eval(response_list[0])
            if len(response_list) > 1:
                get_rules['bl_list'] = response_list[1:]
        else:
            get_rules = ast.literal_eval(response.text)
        if get_rules['status'] == 'fail':
            error_description = get_rules['description']
            logging.error(
                'Get status fail when get {setting} settings for key: {key} with description: {description}'
                .format(setting=module, key=format_key, description=str(error_description))
            )
        if get_rules['status'] == 'none':
            logging.info('Get status none when get {setting} settings for key: {key}'
                         .format(setting=module, key=format_key))
            return None
        if get_rules['status'] == 'success':
            return get_rules
    else:
        logging.info('Response code {code} when get {setting} settings for key: {key}'.
                     format(code=str(response.status_code), setting=module, key=format_key))
        return None


def send_set_settings(module, data) -> tuple:
    """
    Function to set settings
    :param module: Module name
    :param data: Sha256 license key
    :return: Response
    """
    url = ''
    if module == 'MLC':
        url = '/set_mlc_settings'
    if module == 'DYN':
        url = '/set_dyn_settings'
    if module == 'ERL':
        url = '/set_dyn_erl'
    if module == 'WL':
        url = '/set_dyn_wl'

    set_response = requests.post(
        api_url + url,
        headers={'User-agent': USER_AGENT},
        json=data,
        timeout=10
    )
    if set_response.ok:
        api_response = ast.literal_eval(set_response.text)
        if api_response['status'] == 'fail':
            error_description = api_response['description']
            body = json.loads(set_response.request.body.decode('utf-8'))
            body['key'] = '{}{}'.format('X' * 26, body.get('key')[-6:])
            logging.error(
                'Get status fail when set {setting} settings with description: {description} and data: {data}'.format(
                    setting=module, description=str(error_description), data=json.dumps(body)))
            if not error_description:
                error_description = 'Get status fail when set {setting} settings'
            return 'fail', error_description
        else:
            if module == 'WL':
                if 'rid' in api_response:
                    return 'success', api_response['rid']
                else:
                    return 'success', None
            elif module == 'ERL':
                if 'rid' in api_response:
                    return 'success', api_response['rid']
                else:
                    return 'success', None
            else:
                return 'success', None
    else:
        body = json.loads(set_response.request.body.decode('utf-8'))
        body['key'] = '{}{}'.format('X' * 26, body.get('key')[-6:])

        logging.info(
            'Response code {code} when set {setting} settings with data: {data}'.format(
                code=str(set_response.status_code),
                setting=module,
                data=json.dumps(body)))
        return 'fail', "Something's went wrong"


def check_keys(keys, check_waf_id):
    """
    Checks keys and their waf id
    :param keys: List keys
    :param check_waf_id: Condition for checking waf id
    :return: List of checked keys and waf ids
    """
    checked_key = []
    checked_key_ids = []
    waf_ids = []

    processes = []
    checked_key_queue = multiprocessing.Queue()
    checked_key_ids_queue = multiprocessing.Queue()

    for key in keys:
        proc = multiprocessing.Process(target=send_request_check_key,
                                       args=(key, check_waf_id, waf_ids, checked_key_queue, checked_key_ids_queue))
        proc.start()
        processes.append(proc)

    for pr in processes:
        pr.join()

    while not checked_key_queue.empty():
        checked_key.append(checked_key_queue.get())

    while not checked_key_ids_queue.empty():
        checked_key_ids.append(checked_key_ids_queue.get())

    return checked_key, checked_key_ids


def send_request_check_key(key, check_waf_id, waf_ids, checked_key_queue, checked_key_ids_queue):
    """
    Sends a request to check the key
    :param key: License key
    :param check_waf_id:
    :param waf_ids: List waf id
    :param checked_key_queue: Queue with keys
    :param checked_key_ids_queue: Queue with waf ids
    """
    try:
        response_check_key = requests.post(
            auth_url,
            proxies=get_proxy_dict(),
            headers={'User-agent': USER_AGENT},
            data={'key': key.key}
        )

        if response_check_key.ok:
            check_key_result = json.loads(response_check_key.text.replace('\'', '\"'))
            if check_key_result['status'] == 'success' and str(check_key_result['r_asa_disable']) == 'False':
                tarif_id = int(check_key_result['tarif_id'])
                if tarif_id == 0:
                    key.tarif = 'Light'
                elif tarif_id == 1:
                    key.tarif = 'Business'
                elif tarif_id == 2:
                    key.tarif = 'Enterprise'
                if check_waf_id:
                    if check_key_result['waf_id'] not in waf_ids:
                        waf_ids.append(check_key_result['waf_id'])
                        checked_key_queue.put(key)
                        checked_key_ids_queue.put(key.id)
                else:
                    checked_key_queue.put(key)
                    checked_key_ids_queue.put(key.id)
        else:
            logger.info('Response code {code} when checking the key: {key}'.
                        format(code=str(response_check_key.status_code), key=key.get_formated_key()))
    except Exception as e:
        logger.error('An error occurred while confirming the key: {key}. Error information: {err}'
                     .format(key=key.get_formated_key(), err=e))


def get_dyn_settings(key, queue):
    """
    Processes received DYN settings
    :param key: License key
    :param queue: Multiprocessor queue
    """
    getting_dyn_settings = get_settings('DYN', key.key)
    if getting_dyn_settings is not None:
        for dyn_key, value in getting_dyn_settings.items():
            if dyn_key == 'nwaf_ai_extra_host_lm':
                key.nwaf_ai_extra_host_lm = value
            if dyn_key == 'nwaf_ai_extra_host_wl':
                key.nwaf_ai_extra_host_wl = value
            if dyn_key == 'nwaf_bf_detect_host_lm':
                key.nwaf_bf_detect_host_lm = value
            if dyn_key == 'nwaf_body_exclude':
                key.nwaf_body_exclude = value
            if dyn_key == 'nwaf_ddos_detect_host_lm':
                key.nwaf_ddos_detect_host_lm = value
            if dyn_key == 'nwaf_host_lm':
                key.nwaf_host_lm = value
            if dyn_key == 'nwaf_host_wl':
                key.nwaf_host_wl = value
            if dyn_key == 'nwaf_ip_lm':
                key.nwaf_ip_lm = value
            if dyn_key == 'nwaf_ip_wl':
                key.nwaf_ip_wl = value
            if dyn_key == 'nwaf_geoip_mla_disable':
                key.nwaf_geoip_mla_disable = value
            if dyn_key == 'nwaf_b64_decode_disable':
                key.nwaf_b64_decode_disable = value
            if dyn_key == 'nwaf_limit':
                nwaf_limits = str(value).split(', ')
                nwaf_limits_list = []
                for limit in nwaf_limits:
                    limit_dict = {}
                    for el in limit.split(' '):
                        if 'domain' in el:
                            limit_dict['domain'] = el.replace('domain=', '')
                        if 'rate' in el:
                            limit_dict['rate'] = el.replace('rate=', '')
                        if 'block_time' in el:
                            limit_dict['block_time'] = el.replace('block_time=', '')
                    nwaf_limits_list.append(limit_dict)
                key.nwaf_limit = nwaf_limits_list
            if dyn_key == 'nwaf_mla_host_lm':
                key.nwaf_mla_host_lm = value
            if dyn_key == 'nwaf_put_body_exclude':
                key.nwaf_put_body_exclude = value
            if dyn_key == 'nwaf_rmq_host_exclude':
                key.nwaf_rmq_host_exclude = value

            if dyn_key == 'nwaf_openapi_url_wl':
                key.nwaf_openapi_url_wl = value
            if dyn_key == 'nwaf_openapi_ip_wl':
                key.nwaf_openapi_ip_wl = value
            if dyn_key == 'nwaf_openapi_url_lm':
                key.nwaf_openapi_url_lm = value
            if dyn_key == 'nwaf_openapi_ip_lm':
                key.nwaf_openapi_ip_lm = value
            if dyn_key == 'nwaf_post_body_exclude':
                key.nwaf_post_body_exclude = value
            if dyn_key == 'nwaf_url_wl':
                key.nwaf_url_wl = value

    queue.put(key)


def get_mlc_settings(key, queue):
    """
    Processes received MLC settings
    :param key: License key
    :param queue: Multiprocessor queue
    """
    response_list_vhost = requests.post(
        api_url + '/get_vhosts_list',
        headers={'User-agent': USER_AGENT},
        data={'key': key.key},
    )
    if response_list_vhost.ok:
        response_list_vhost_dict = ast.literal_eval(response_list_vhost.text)
        try:
            res_list_vhost = str(response_list_vhost_dict['vhosts_list']).strip('][').replace('\"', '')
            tmp_list_vhost = ''
            for num, el in enumerate(res_list_vhost.split(',')):
                tmp_list_vhost += convert_from_punycode(el)
                if num != len(res_list_vhost.split(',')) - 1:
                    tmp_list_vhost += ','
            key.vhost_list = tmp_list_vhost
        except KeyError:
            key.vhost_list = ''
            logger.error('Error with get vhost lists: ' + key.get_formated_key())
    else:
        key.vhost_list = ''
        logger.info(
            'Response code {code} when get vhosts list: {key}'.format(code=str(response_list_vhost.status_code),
                                                                      key=key.get_formated_key()))

    getting_mlc_settings = get_settings('MLC', key.key)
    if getting_mlc_settings is not None:
        for setting_key, value in getting_mlc_settings.items():
            if setting_key == 'main__ai_extra':
                key.main__ai_extra = value
            if setting_key == 'ddos__enable':
                key.ddos__enable = value
            if setting_key == 'ddos__wl_ip':
                key.ddos__wl_ip = str(value).replace(' ', ', ')
            if setting_key == 'ddos__wl_url':
                key.ddos__wl_url = str(value).strip('][').replace('\"', '')
            if setting_key == 'ddos__interval':
                key.ddos__interval = value
            if setting_key == 'ddos__latest_only':
                key.ddos__latest_only = value
            if setting_key == 'ddos__send_possible':
                key.ddos__send_possible = value

            if setting_key == 'brute__enable':
                key.brute__enable = value
            if setting_key == 'brute__brute_detect':
                key.brute__brute_detect = str(value).strip('][').replace('\"', '')
            if setting_key == 'brute__flood_detect':
                key.brute__flood_detect = str(value).strip('][').replace('\"', '')
            if setting_key == 'brute__wl_host':
                key.brute__wl_host = str(value).strip('][').replace('\"', '')
            if setting_key == 'brute__interval':
                key.brute__interval = value
            if setting_key == 'brute__max_val':
                key.brute__max_val = value
            if setting_key == 'brute__latest_only':
                key.brute__latest_only = value
            if setting_key == 'brute__send_possible':
                key.brute__send_possible = value

            if setting_key == 'openapi__vhosts_list':
                key.openapi__vhosts_list = str(value).strip('][').replace('\"', '')

    response_list_models = requests.post(
        api_url + '/get_models_list_uri',
        headers={'User-agent': USER_AGENT},
        data={'key': key.key},
        timeout=10
    )
    if response_list_models.ok:
        tmp_list_models = response_list_models.text[1:-1].replace('\'', '').replace('sub_', '*.').split(', ')

        key.models = []

        for i in tmp_list_models:
            key.models.append(convert_from_punycode(i))

    else:
        key.models = []
        logger.info(
            'Response code {code} when get models list: {key}'
            .format(
                code=str(response_list_models.status_code),
                key=key.get_formated_key()
            )
        )

    model_list = []
    try:
        vhost_list = [i.strip() for i in key.vhost_list.split(', ')]
        for md in key.models:
            vhost = None

            if md in vhost_list:
                vhost = md
            elif md.lower() == 'all' and '*' in vhost_list:
                vhost = '*'
            else:
                for i in vhost_list:
                    if i.startswith('.') and (i in md or i[1:] == md):
                        vhost = i
                        break

            vhost = convert_to_punycode(vhost)
            response_training_status = requests.post(
                api_url + '/get_training_uri',
                headers={'User-agent': USER_AGENT},
                data={'key': key.key,
                      'vhost': vhost},
                timeout=10
            )

            if response_training_status.ok:
                response_training_status_dict = ast.literal_eval(response_training_status.text)
                if response_training_status_dict['status'] == 'success':
                    model = {
                        'name': md,
                        'duration': response_training_status_dict['duration'],
                        'complete': str(response_training_status_dict['complete']),
                    }
                    if str(response_training_status_dict['progress']).isdigit():
                        model['progress'] = str(response_training_status_dict['progress'])
                    else:
                        model['progress'] = 'none'
                else:
                    logger.info(
                        'Get status {status} when get training model {model} with key: {key}'.format(
                            status=str(response_training_status_dict['status']), model=md, key=key.get_formated_key()))
                    model = {
                        'name': md,
                        'duration': 'none',
                        'complete': 'none',
                        'progress': 'none'
                    }
                model_list.append(model)
            else:
                logger.info(
                    'Response code {code} when get training model {model} with key: {key}'.format(
                        code=str(response_training_status.status_code), model=md, key=key.get_formated_key()))
    except Exception as e:
        logger.error('Error with get models with: {exception}'.format(exception=str(e)))
    key.models = model_list

    queue.put(key)


def get_lic_keys(key, queue):
    """
    Processes received license key
    :param key: License key
    :param queue: Multiprocessor queue
    """
    try:
        response_check_key = requests.post(
            auth_url,
            proxies=get_proxy_dict(),
            headers={'User-agent': USER_AGENT},
            data={'key': key.key}
        )
        if response_check_key.ok:
            check_key_result = json.loads(response_check_key.text.replace('\'', '\"'))
            if check_key_result['status'] == 'success':
                key.remote_asa_disable = str(check_key_result['r_asa_disable'])
                key.models_num = int(check_key_result['models_num'])
                key.days_left = str(check_key_result['days_left']).replace('days', '')
                key.waf_id = str(check_key_result['waf_id'])
                tarif_id = int(check_key_result['tarif_id'])
                if tarif_id == 0:
                    key.tarif = 'Light'
                elif tarif_id == 1:
                    key.tarif = 'Business'
                elif tarif_id == 2:
                    key.tarif = 'Enterprise'
                queue.put(key)
        else:
            logger.info('Response code {code} when checking the key: {key}'.
                        format(code=str(response_check_key.status_code), key=key.get_formated_key()))
    except Exception as e:
        logger.error('An error occurred while processing the key: {key}. Error information: {err}'
                     .format(key=key.get_formated_key(), err=e))


def get_rules_management(key, queue):
    """
    Processes received ERL and WL rules and BL
    :param key: License key
    :param queue: Multiprocessor queue
    """
    get_erl = []
    getting_erl_settings = get_settings('ERL', key.key)
    if getting_erl_settings is not None:
        for el in getting_erl_settings:
            get_erl.append(ast.literal_eval(str(el).replace('true', 'True').replace('false', 'False')))
        for erl in get_erl:
            for erl_key, erl_value in erl.items():
                try:
                    if erl_key == 'other_headers':
                        other_headers = []
                        for el in erl_value:
                            name = el[:el.find(':')]
                            value = el[el.find(':') + 1:]
                            if '!!' in name:
                                name = name.replace('!!', '')
                                name = base64.b64decode(str(name).encode('ascii')).decode('ascii')
                                name = '!!' + name
                            else:
                                name = base64.b64decode(str(name).encode('ascii')).decode('ascii')
                            if '|' in value:
                                value = re.split(r'(?<!\\)\|', value)
                                for value_index, val in enumerate(value):
                                    value[value_index] = base64.b64decode(str(val).encode('ascii')).decode('ascii')
                                value = '|'.join(value)
                            elif '&' in value:
                                value = re.split(r'(?<!\\)&', value)
                                for value_index, val in enumerate(value):
                                    value[value_index] = base64.b64decode(str(val).encode('ascii')).decode('ascii')
                                value = '&'.join(value)
                            elif '!' in value:
                                value = value.replace('!', '')
                                value = base64.b64decode(str(value).encode('ascii')).decode('ascii')
                                value = '!' + value
                            else:
                                value = base64.b64decode(str(value).encode('ascii')).decode('ascii')
                            value = value.replace(':', '\\:')
                            value = value.replace('"', '\\"')
                            new_value = value
                            res = list(
                                filter(None,
                                       new_value.replace('|!', ',|!').replace('|&', ',|&').replace('|', ',|').replace(
                                           '&', ',&').
                                       replace('!', '!').split(',')))

                            result = []
                            for element in res:
                                if element[0] == '|' and element[1] == '!':
                                    result.append(['|!', element[2:]])
                                elif element[0] == '&' and element[1] == '!':
                                    result.append(['&!', element[2:]])
                                elif element[0] == '|' and element[1] != '!':
                                    result.append(['|', element[1:]])
                                elif element[0] == '&' and element[1] != '!':
                                    result.append(['&', element[1:]])
                                elif element[0] == '!':
                                    result.append(['!', element[1:]])
                                else:
                                    result.append([' ', element])
                            header = {'key': name, 'value': result}
                            other_headers.append(header)
                        erl[erl_key] = other_headers
                    else:
                        erl_value = str(erl_value)
                        erl[erl_key] = list(
                            filter(None, erl_value.replace('|!', ',|!').replace('|&', ',|&').replace('|', ',|')
                                   .replace('&', ',&').replace('!', '!').split(',')))
                        result = []
                        for el in erl[erl_key]:
                            if el[0] == '|' and el[1] == '!':
                                result.append(['|!', base64.b64decode(str(el[2:]).encode('ascii')).decode('ascii')])
                            elif el[0] == '&' and el[1] == '!':
                                result.append(['&!', base64.b64decode(str(el[2:]).encode('ascii')).decode('ascii')])
                            elif el[0] == '|' and el[1] != '!':
                                result.append(['|', base64.b64decode(str(el[1:]).encode('ascii')).decode('ascii')])
                            elif el[0] == '&' and el[1] != '!':
                                result.append(['&', base64.b64decode(str(el[1:]).encode('ascii')).decode('ascii')])
                            elif el[0] == '!':
                                result.append(['!', base64.b64decode(str(el[1:]).encode('ascii')).decode('ascii')])
                            else:
                                result.append([' ', base64.b64decode(str(el).encode('ascii')).decode('ascii')])
                        if len(result) > 0:
                            erl[erl_key] = result
                        else:
                            erl[erl_key] = ''
                except Exception:
                    erl[erl_key] = erl_value
        get_erl = sorted(get_erl, key=lambda d: d['id'], reverse=True)
        key.get_erl = get_erl

    getting_wl_rules = get_settings('WL', key.key)

    get_rules = []
    if getting_wl_rules is not None:
        for rule in getting_wl_rules:

            get_rules.append(rule)

    getting_bl_rules = get_settings('BL', key.key)
    if getting_bl_rules is not None:
        bl_list = getting_bl_rules.get('bl_list', list())
        bl_dict = dict()
        for i in bl_list:
            if 'domain=' in i:
                domain = i.split(' ')[0].replace('domain=', '')
                ip = i.split(' ')[1]
            else:
                domain = '__ALL__'
                ip = i
            ip = ip.strip()
            domain = domain.strip()
            if bl_dict.get(domain) is None:
                bl_dict[domain] = list()
            bl_dict[domain].append(ip)

        key.bl_all = bl_dict.get('__ALL__')
        key.bl_all_count = len(bl_dict.get('__ALL__', []))
        if '__ALL__' in bl_dict:
            del bl_dict['__ALL__']
        key.bl_dicts = bl_dict
        key.bl_dicts_count = dict()
        for k, v in bl_dict.items():
            key.bl_dicts_count[k] = len(v)

    list_wl_dicts = []
    if get_rules is not None:
        for el in get_rules:
            rule_dict = {}
            try:
                rule_dict['rid'] = str(re.findall(r'RID: \d+', el)[0]).replace('RID: ', '')
            except Exception:
                rule_dict['rid'] = ''
                logger.error('Error with parsing RID in rule:\n' + el)

            try:
                rule_dict['wl_id'] = str(re.findall(r'WL ID:\d+|\*', el)[0]).replace('WL ID:', '')
            except Exception:
                rule_dict['wl_id'] = ''
                logger.error('Error with parsing WL ID in rule:\n' + el)

            try:
                rule_dict['active'] = str(re.findall(r'Active: True|False', el)[0]).replace('Active: ', '')
            except Exception:
                rule_dict['active'] = ''
                logger.error('Error with parsing Active in rule:\n' + el)

            domain = re.findall(r'(domain=)([-a-zA-Z\d@:%_+.~#?&/=]{2,256})', el)

            if len(domain) > 0:
                rule_dict['domain'] = str(re.findall(r'(domain=)([-a-zA-Z\d@:%_+.~#?&/=]{2,256})', el)[0][1])
                rule_dict['domain'] = convert_from_punycode(rule_dict['domain'])

            mz_and_extension = re.findall(r'Z:(.+)\"', el)
            mz_and_extension = str(mz_and_extension[0])
            find_mz = mz_and_extension.find('$')
            if find_mz == -1:
                rule_dict['mz'] = str(mz_and_extension).rstrip('|').upper()
                rule_dict['mz'] = rule_dict['mz'].replace('COOKIE', 'Cookie')
                rule_dict['extension'] = ''
            else:
                rule_dict['mz'] = mz_and_extension[:-(len(mz_and_extension) - find_mz + 1)]
                rule_dict['mz'] = rule_dict['mz'].upper()
                rule_dict['mz'] = rule_dict['mz'].replace('COOKIE', 'Cookie')
                extension = mz_and_extension[find_mz:]
                extension_str = re.sub(r'\|\$', '|', extension)
                rule_dict['extension'] = extension_str[1:]
                extension = extension.split('|$')
                extension_dict = {}
                for ext in extension:
                    extension_dict[ext[:ext.rfind(':')].replace('$', '')] = ext[ext.find(':') + 1:]
                rule_dict['extension_dict'] = extension_dict

            list_wl_dicts.append(rule_dict)
            list_wl_dicts = sorted(list_wl_dicts, key=lambda d: d['rid'], reverse=True)
            key.wl_dicts = list_wl_dicts
    else:
        key.wl_dicts = list_wl_dicts

    queue.put(key)


def set_default_values(hashed_key):
    """
    Sets default values for MLC and Dyn when new key is added
    :param hashed_key: Hashed license key
    """
    mlc_settings_dict = {
        'ddos__enable': ddos.enable,
        'ddos__interval': ddos.interval,
        'ddos__latest_only': ddos.latest_only,
        'ddos__send_possible': ddos.send_possible,
        'brute__enable': brute.enable,
        'brute__interval': brute.interval,
        'brute__max_val': brute.max_val,
        'brute__latest_only': brute.latest_only,
        'brute__send_possible': brute.send_possible,
    }
    mlc_full_settings_dict = {"key": hashed_key, "set": mlc_settings_dict}
    send_set_settings('MLC', mlc_full_settings_dict)

    dyn_settings_dict = {
        'nwaf_limit': nwaf_limit.nwaf_limit
    }
    dyn_full_settings_dict = {"key": hashed_key, "set": dyn_settings_dict}
    send_set_settings('DYN', dyn_full_settings_dict)


def get_erl_rule(key, erl_id):
    get_erl = []
    getting_erl_settings = get_settings('ERL', key.key)
    if getting_erl_settings is not None:
        for el in getting_erl_settings:
            get_erl.append(ast.literal_eval(str(el).replace('true', 'True').replace('false', 'False')))
        for erl in get_erl:
            if erl['id'] == int(erl_id):
                return erl


def set_dyn_bl(key, list_bl):
    get_bl = get_settings('BL', key.key)
    if get_bl:
        general_list = get_bl.get('bl_list', list())
        list_dupl = set()
        for i in general_list.copy():

            if i in list_bl:
                list_dupl.add(i)
            else:
                api_in_i = 'api=' in i
                domain_in_i = i.split(' ')[0] if 'domain=' in i else None

                if 'domain=' in i:
                    i_tmp = i.split(' ')[1]
                elif 'api=' in i:
                    i_tmp = i.split(' ')[0]
                else:
                    i_tmp = i

                for j in list_bl:
                    api_in_j = 'api=' in j
                    domain_in_j = j.split(' ')[0] if 'domain=' in j else None

                    if 'domain=' in j:
                        j_tmp = j.split(' ')[1]
                    elif 'api=' in j:
                        j_tmp = j.split(' ')[0]
                    else:
                        j_tmp = j

                    if i_tmp == j_tmp:
                        if (not domain_in_j and domain_in_i and api_in_i == api_in_j) \
                                or (domain_in_j == domain_in_i and api_in_i != api_in_j):
                            if i in general_list:
                                general_list.remove(i)
                        elif domain_in_j and domain_in_i and domain_in_j == domain_in_i and api_in_i == api_in_j:
                            list_dupl.add(j)
                        elif domain_in_j and not domain_in_i:
                            list_dupl.add(j)

        list_bl += general_list
        list_dupl = list(list_dupl)
        for i in list_dupl:
            list_bl.remove(i)
    else:
        list_dupl = []

    data = {
        "key": key.key,
        "set": {
            "active": "true",
            "bl": list_bl
        }
    }
    response = requests.post(
        api_url + '/set_dyn_bl',
        headers={'User-agent': USER_AGENT},
        json=data,
    )

    if response.ok:
        d = json.loads(response.text.replace('\'', '\"'))
        if d['status'] == 'success':
            return list_bl, list_dupl
        else:
            logger.info('An error occurred while set settings BL with {key} and description error: {desc}'.
                        format(code=str(response.status_code), key=key.get_formated_key(), desc=d['description']))
            return False, d['description']
    else:
        logger.info('Response code {code} when set blocked list for key: {key}'.
                    format(code=str(response.status_code), key=key.get_formated_key()))
        return False, 'Unknown error'


def get_dyn_bl(key, search):
    get_bl = get_settings('BL', key.key)
    if get_bl:
        get_bl = get_bl.get('bl_list', list())
    else:
        get_bl = []

    if search:
        res_list = list()
        for i in search:
            if not i:
                continue
            for j in get_bl:
                if 'domain=' in j:
                    ip_j = j.split(' ')[1]
                elif 'api=' in j:
                    ip_j = j.split(' ')[0]
                else:
                    ip_j = j

                if '/' in i:
                    ip_net = IPNetwork(i)
                    if '/' in ip_j:
                        ip_net2 = IPNetwork(ip_j)
                        if ip_net2 in ip_net or ip_net in ip_net2:
                            res_list.append(j)
                    elif '-' in ip_j:
                        ip_range = IPRange(ip_j.split('-')[0], ip_j.split('-')[1]).cidrs()

                        for c in ip_range:
                            if c in ip_net or ip_net in c:
                                res_list.append(j)
                                break
                    else:
                        ip = IPAddress(ip_j)
                        if ip in ip_net:
                            res_list.append(j)
                elif '-' in i:
                    try:
                        ip_range = IPRange(i.split('-')[0], i.split('-')[1]).cidrs()
                    except (AddrFormatError, AddrConversionError):
                        if i in ip_j or ip_j in i:
                            res_list.append(j)
                        continue
                    if '/' in ip_j:
                        ip_net2 = IPNetwork(ip_j)
                        for c in ip_range:
                            if ip_net2 in c or c in ip_net2:
                                res_list.append(j)
                                break
                    elif '-' in ip_j:
                        ip_range2 = IPRange(ip_j.split('-')[0], ip_j.split('-')[1]).cidrs()

                        for c in ip_range:
                            for m in ip_range2:
                                if c in m or m in c:
                                    res_list.append(j)
                                    break
                    else:
                        ip = IPAddress(ip_j)
                        for c in ip_range:
                            if ip in c:
                                res_list.append(j)
                                break
                else:
                    try:
                        ip = IPAddress(i)
                    except (AddrFormatError, AddrConversionError):
                        if i in ip_j or ip_j in i:
                            res_list.append(j)
                        continue
                    if '/' in ip_j:
                        ip_net = IPNetwork(ip_j)
                        if ip in ip_net:
                            res_list.append(j)
                    elif '-' in ip_j:
                        ip_range = IPRange(ip_j.split('-')[0], ip_j.split('-')[1]).cidrs()
                        for c in ip_range:
                            if ip in c:
                                res_list.append(j)
                                break
                    else:
                        ip2 = IPAddress(ip_j)
                        if ip == ip2:
                            res_list.append(j)
        return list(set(res_list))
    else:
        return get_bl


def del_dyn_bl(key, search, mode=0):
    get_bl = get_settings('BL', key.key)

    if get_bl:
        bl_list = get_bl.get('bl_list', list())
        if mode == 0:
            for i in search:
                bl_list.remove(i)
        elif mode == 1:
            rem_list = [i for i in get_bl.get('bl_list', list()) if search in i]
            for i in rem_list:
                bl_list.remove(i)
        else:
            return False
    else:
        return []

    data = {
        "key": key.key,
        "set": {
            "active": "true",
            "bl": bl_list
        }
    }
    response = requests.post(
        api_url + '/set_dyn_bl',
        headers={'User-agent': USER_AGENT, 'Content-Type': 'application/json'},
        json=data,
    )
    if response.ok:
        d = json.loads(response.text.replace('\'', '\"'))
        if d['status'] == 'success':
            return bl_list
        else:
            return False
    else:
        logger.info('Response code {code} when del ip in blocked list for key: {key}'.
                    format(code=str(response.status_code), key=key.get_formated_key()))
        return False


def sort_bl(bl_list):
    res = list()
    for type_ip in ('-', '/', 'only'):
        tmp = list()
        for i in sorted(bl_list):
            if type_ip == 'only':
                t = [p if len(p) == 3 else '0' * (3 - len(p)) + p for p in i.replace('__ALL__', '').split('.')]
                num = ''.join(t)
                tmp.append({'num': num,
                            'ip': i})
            elif type_ip in i:
                tmp.append(i)
        for i in tmp:
            if type_ip == 'only':
                bl_list.remove(i['ip'])
            else:
                bl_list.remove(i)
        if type_ip == 'only':
            for ip in sorted(tmp, key=lambda x: x['num']):
                res.append(ip['ip'])
        else:
            res += tmp

    return res
