import csv
import logging

from django.conf import settings
from django.http import HttpResponse

from main.models import ScannerReport
from main.utils import convert_from_punycode
from report.utils.counters import AttackCounter, make_attack_type_counters, make_extra_counters
from report.utils.utils import get_end_datetime, \
    get_start_datetime, date_to_str, date_to_str_with_time, get_filtered_attacks, get_top_ip_list

logger = logging.getLogger(__name__)


def generate_csv_report(request):
    start_date = get_start_datetime(request)
    end_date = get_end_datetime(request)

    start_date_str = date_to_str(start_date)
    end_date_str = date_to_str(end_date)
    logger.info(f'Start generate CSV report {start_date_str}-{end_date_str}')

    response = HttpResponse(
        content_type='text/csv',
        headers={'Content-Disposition': f'attachment; filename="report({start_date_str} - {end_date_str}).csv"'},
    )

    writer = csv.writer(response, delimiter=";")

    attacks_logs = get_filtered_attacks(start_date, end_date, user_id=request.user.id)

    write_report_title(writer, attacks_logs, start_date_str, end_date_str)
    write_all_counters(writer, attacks_logs, start_date, end_date)
    find_and_write_top_ip(writer, attacks_logs)
    write_all_attacks(writer, attacks_logs)
    logger.info(f'CSV report generated! {start_date_str}-{end_date_str}')
    return response


def write_all_attacks(writer, attacks_logs):
    for attack in attacks_logs.order_by('timestamp').iterator():
        attack_time = date_to_str_with_time(attack.get('timestamp'))

        mz = str(attack.get('mz'))
        if mz == 'URL':
            mz = 'URI'

        bt = attack.get('bt')
        attack_info = f'Blocked by {settings.BT_DICT.get(bt).description}'
        if mz:
            attack_info += f' in {mz}'

        attack_method = attack.get('method').strip()
        vhost = convert_from_punycode(attack.get('vhost'))
        ip = attack.get('ip').strip()
        request_info = f"Target: {vhost} \n" \
                       f"Source IP: {ip}"
        if attack_method:
            request_info += f" ({attack_method})"

        writer.writerow([attack_time, attack_info, request_info])


def find_and_write_top_ip(writer, attacks_logs):
    top_ip = get_top_ip_list(attacks_logs)
    writer.writerow([f'Attackers Top {len(top_ip)}'])
    writer.writerow(['IP-address', 'Count', 'Country'])
    for ip_dict in top_ip:
        writer.writerow([ip_dict['ip'], ip_dict['ip__count'], ip_dict['country']])
    writer.writerow([])


def write_all_counters(writer, attacks_logs, start_date, end_date):
    counters = make_attack_type_counters(attacks_logs)
    write_counters_on_title(writer, counters)
    writer.writerow([])
    extra_counters = make_extra_counters(attacks_logs)
    write_counters_on_title(writer, extra_counters)
    count_and_write_scanner(writer, end_date, start_date)
    writer.writerow([])


def count_and_write_scanner(writer, end_date, start_date):
    scanner_count = ScannerReport.objects.using('waf').all().filter(scan_date__gte=start_date,
                                                                    scan_date__lte=end_date).count()
    if scanner_count:
        writer.writerow([settings.SCANNER_DESCRIPTION, scanner_count])


def write_counters_on_title(writer, counters):
    for counter in counters:
        counter: AttackCounter
        if counter.numbers_of_attacks:
            writer.writerow([counter.description, counter.numbers_of_attacks])


def write_report_title(writer, attacks_logs, start_date_str, end_date_str):
    logs_count_all = attacks_logs.count()
    writer.writerow(['Nemesida WAF'])
    writer.writerow(['Website Protection Summary Report'])
    writer.writerow(
        ['{start_date} - {end_date}'.format(start_date=start_date_str, end_date=end_date_str)])
    writer.writerow(['Nemesida WAF blocked', logs_count_all])
    writer.writerow([])
